import { Observable } from '@nativescript/core';
import { JSONfn } from './JSONfn';
export * from './defineWorker';
/**
 * List of task states.
 */
export var TaskStatus;
(function (TaskStatus) {
    TaskStatus[TaskStatus["Starting"] = 0] = "Starting";
    /**
     * The task has been initialized but has not yet been invoked.
     */
    TaskStatus[TaskStatus["Created"] = 1] = "Created";
    /**
     * The task completed due to an unhandled exception.
     */
    TaskStatus[TaskStatus["Faulted"] = 2] = "Faulted";
    /**
     * The task completed execution successfully.
     */
    TaskStatus[TaskStatus["RanToCompletion"] = 3] = "RanToCompletion";
    /**
     * The task is running but has not yet completed.
     */
    TaskStatus[TaskStatus["Running"] = 4] = "Running";
    /**
     * The task has been scheduled for execution but has not yet begun executing.
     */
    TaskStatus[TaskStatus["WaitingToRun"] = 5] = "WaitingToRun";
})(TaskStatus || (TaskStatus = {}));
let dataGlobalWorker = { worker: null, isGlobal: true, running: false, globalResolve: {}, globalReject: {}, globalOnUpdate: {} };
let globalConfiguration = { stickyWorker: true, newWorkerIfGlobalIsUsed: true, startGlobalWorker: true };
/**
 * A task.
 */
export class Task extends Observable {
    /**
     * Initializes a new instance of that class.
     *
     * @param {TaskFunc<TState, TResult>} func The function to invoke.
     */
    constructor(func, funcUptade) {
        super();
        /**
         * Stores the current task state.
         */
        this._status = TaskStatus.Starting;
        this.dataWorker = { worker: null, isGlobal: false, running: false };
        this.id = '';
        if (func !== null && func !== undefined) {
            if (typeof func !== 'function') {
                throw "'func' must be a function!";
            }
        }
        this._FUNC = func;
        this._FUNC_UPDATE = funcUptade;
        this.updateStatus(TaskStatus.Created);
    }
    static globalWorkerConfig(options) {
        globalConfiguration = { ...globalConfiguration, ...options };
        if (globalConfiguration.startGlobalWorker) {
            Task.initGlobalWorker();
        }
    }
    static initGlobalWorker() {
        if (dataGlobalWorker.worker == null) {
            dataGlobalWorker.worker = new Worker('~/globalWorker');
        }
        return dataGlobalWorker.worker;
    }
    newWorker() {
        return new Worker('~/globalWorker');
    }
    static finishGlobalWorker() {
        dataGlobalWorker.worker?.terminate();
        dataGlobalWorker.worker = null;
        dataGlobalWorker.running = false;
    }
    /**
     * Creates a new task.
     *
     * @param {TaskFunc<TResult>} func The function to invoke.
     *
     * @return {Task<TResult>} The new task.
     */
    static newTask(func, onProgressUpdate) {
        return new Task(func, onProgressUpdate);
    }
    /**
     * Creates and starts a new task.
     *
     * @param {TaskFunc<TResult>} func The function to invoke.
     * @param {TState} [state] The optional value / object for the execution.
     *
     * @return {Promise<TResult>} The promise.
     */
    static start(func, options) {
        return Task.newTask(func, options?.onProgressUpdate).start(options);
    }
    /**
     * Gets the error of the last execution.
     */
    get error() {
        return this._error;
    }
    /**
     * Gets the underyling function to invoke.
     */
    get func() {
        return this._FUNC;
    }
    /**
     * Gets the underyling upudate function to invoke.
     */
    get funcUpdate() {
        return this._FUNC_UPDATE;
    }
    getWorker() {
        if (globalConfiguration.newWorkerIfGlobalIsUsed && dataGlobalWorker.running) {
            return { worker: this.newWorker(), isGlobal: false, running: false };
        }
        if (dataGlobalWorker.worker === null) {
            dataGlobalWorker.worker = this.newWorker();
            return { worker: dataGlobalWorker.worker, isGlobal: true, running: false };
        }
        else {
            return { worker: dataGlobalWorker.worker, isGlobal: true, running: false };
        }
    }
    /**
     * Starts the task.
     *
     * @param {TState} [state] The optional value to submit to the function.
     *
     * @return {Promise<TaskResult<TState, TResult>>} The promise.
     */
    start(options) {
        this.id = Date.now().toString();
        let me = this;
        return new Promise((resolve, reject) => {
            let completed = (id, err, data) => {
                if (err) {
                    const rejectFunction = me.getReject(id, reject);
                    if (rejectFunction) {
                        rejectFunction({
                            error: err,
                            state: options?.state,
                        });
                    }
                }
                else {
                    const resolveFunction = me.getResolve(id, resolve);
                    if (resolveFunction) {
                        resolveFunction({
                            data: data,
                            state: options?.state,
                        });
                    }
                }
                if (options?.performance) {
                    console.timeEnd('RunWorker');
                }
                me.removeGlobalCallback(id);
                me.checkTerminateWorker(data);
                me.updateError(err);
            };
            switch (me._status) {
                case TaskStatus.Created:
                case TaskStatus.Faulted:
                case TaskStatus.RanToCompletion:
                    break;
                default:
                    completed('', new Error(`Cannot start while in '${TaskStatus[me._status]}' state!`));
                    return;
            }
            try {
                me.updateStatus(TaskStatus.WaitingToRun);
                me.dataWorker = me.getWorker();
                const worker = me.dataWorker.worker;
                me.dataWorker.running = true;
                if (me.dataWorker.isGlobal && dataGlobalWorker.globalResolve && dataGlobalWorker.globalReject && dataGlobalWorker.globalOnUpdate) {
                    dataGlobalWorker.running = true;
                    dataGlobalWorker.globalResolve[me.id] = resolve;
                    dataGlobalWorker.globalReject[me.id] = reject;
                    dataGlobalWorker.globalOnUpdate[me.id] = me._FUNC_UPDATE;
                }
                worker.onmessage = function (msg) {
                    let message = msg.data;
                    let executionId = '';
                    if (message) {
                        message = JSON.parse(message);
                        executionId = message.id;
                    }
                    if (message?.onProgressUpdate === true) {
                        const onUpdateFuncion = me.getOnUpdate(executionId);
                        if (onUpdateFuncion) {
                            onUpdateFuncion({ data: message.dataUpdate });
                        }
                    }
                    else {
                        me.updateStatus(TaskStatus.RanToCompletion);
                        completed(executionId, null, message.result);
                    }
                };
                worker.onerror = function (err) {
                    const idExecution = JSON.parse(err.message.replace('Uncaught Error: ', ''))?.id;
                    me.updateStatus(TaskStatus.Faulted);
                    completed(idExecution, err);
                };
                let func;
                if (me._FUNC) {
                    func = {};
                    func.body = JSONfn.stringify(me._FUNC);
                    if (options?.attachToContextFunctions && Object.keys(options?.attachToContextFunctions).length > 0) {
                        // @ts-ignore
                        const functionsToAttach = Object.keys(options.attachToContextFunctions).reduce((resultObject, keyFunction) => {
                            // @ts-ignore
                            resultObject[keyFunction] = JSONfn.stringify(options.attachToContextFunctions[keyFunction]);
                            return resultObject;
                        }, {});
                        func.attachToContextFunctions = functionsToAttach;
                    }
                }
                if (options?.performance) {
                    console.time('RunUI');
                    // @ts-ignore
                    me._FUNC({ state: options?.state, ...options?.attachToContextFunctions });
                    console.timeEnd('RunUI');
                    console.time('RunWorker');
                }
                me.updateStatus(TaskStatus.Running);
                worker.postMessage(JSON.stringify({
                    id: me.id,
                    func: func,
                    state: options?.state,
                }));
            }
            catch (e) {
                console.log(e);
                me.updateStatus(TaskStatus.Faulted);
                completed(me.id, e);
            }
        });
    }
    getReject(id, reject) {
        if (this.dataWorker.isGlobal) {
            return dataGlobalWorker.globalReject ? dataGlobalWorker.globalReject[id] : null;
        }
        return reject;
    }
    getResolve(id, resolve) {
        if (this.dataWorker.isGlobal) {
            return dataGlobalWorker.globalResolve ? dataGlobalWorker.globalResolve[id] : null;
        }
        return resolve;
    }
    getOnUpdate(id) {
        if (this.dataWorker.isGlobal) {
            return dataGlobalWorker.globalOnUpdate ? dataGlobalWorker.globalOnUpdate[id] : null;
        }
        return this._FUNC_UPDATE;
    }
    removeGlobalCallback(id) {
        if (dataGlobalWorker.globalResolve && dataGlobalWorker.globalResolve[id])
            delete dataGlobalWorker.globalResolve[id];
        if (dataGlobalWorker.globalReject && dataGlobalWorker.globalReject[id])
            delete dataGlobalWorker.globalReject[id];
        if (dataGlobalWorker.globalOnUpdate && dataGlobalWorker.globalOnUpdate[id])
            delete dataGlobalWorker.globalOnUpdate[id];
    }
    checkTerminateWorker(result) {
        if ((!this.dataWorker.isGlobal && result?.onProgressUpdate !== true) || (!globalConfiguration.stickyWorker && dataGlobalWorker.isGlobal && dataGlobalWorker.globalResolve && Object.keys(dataGlobalWorker.globalResolve).length === 0)) {
            this.dataWorker.worker?.terminate();
            this.dataWorker.worker = null;
            if (this.dataWorker.isGlobal) {
                Task.finishGlobalWorker();
            }
        }
        if (result?.onProgressUpdate !== true) {
            this.dataWorker.running = false;
            if (this.dataWorker.isGlobal && dataGlobalWorker.globalResolve && Object.keys(dataGlobalWorker.globalResolve).length === 0)
                dataGlobalWorker.running = false;
        }
    }
    /**
     * Gets the current status.
     */
    get status() {
        return this._status;
    }
    /**
     * Updates the error value.
     *
     * @param {TaskStatus} newValue The new value.
     *
     * @return {boolean} Property change has been raised for 'error' property or not.
     */
    updateError(newValue) {
        if (newValue !== this._error) {
            this._error = newValue;
            this.notifyPropertyChange('error', newValue);
            return true;
        }
        return false;
    }
    /**
     * Updates the current status.
     *
     * @param {TaskStatus} newValue The new value.
     *
     * @return {boolean} Property change has been raised for 'status' property or not.
     */
    updateStatus(newValue) {
        if (newValue !== this._status) {
            this._status = newValue;
            this.notifyPropertyChange('status', newValue);
            return true;
        }
        return false;
    }
}
//# sourceMappingURL=index.js.map